from django.shortcuts import render
from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.contrib import auth
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt
import json
from djangoChat.models import Message, ChatUser
from django.contrib.auth.models import User
import datetime
from django.utils.timezone import now as utcnow


def index(request):
	if request.method == 'POST':
		print request.POST
	logged_users = []
	if request.user.username and request.user.profile.is_chat_user:
		context = {'logged_users':logged_users}
		return render(request, 'djangoChat/index.html', context)
	else:
		return HttpResponseRedirect(reverse('login'))

def login(request):
	if request.user.username and request.user.profile.is_chat_user:
		return HttpResponseRedirect(reverse('index'))
	context = {'error':''}	

	if request.method == 'POST':
		username = request.POST.get('username','') #retunr '' if no username
		password = request.POST.get('password','')

		user = auth.authenticate(username=username,password=password)

		if user is not None:
			auth.login(request,user)
			cu = request.user.profile
			cu.is_chat_user = True
			cu.last_accessed = utcnow()
			cu.save()
			


			return HttpResponseRedirect(reverse('index'))
		else:
			context['error'] = ' wrong credentials try again'
			return render(request,'djangoChat/login.html',context)


	context.update(csrf(request))		
	return render(request,'djangoChat/login.html',context)


def logout(request):
	cu = request.user.profile
	cu.is_chat_user = False
	cu.save()
	return HttpResponse('succesfully logged out of chat')

@csrf_exempt
def chat_api(request):
	if request.method == 'POST':
		d = json.loads(request.body)
		msg =  d.get('msg')
		user = request.user.username
		gravatar = request.user.profile.gravatar_url
		m = Message(user=user,message=msg,gravatar=gravatar)
		m.save()


		res = {'id':m.id,'msg':m.message,'user':m.user,'time':m.time.strftime('%I:%M:%S %p').lstrip('0'),'gravatar':m.gravatar}
		data = json.dumps(res)
		return HttpResponse(data,content_type="application/json")


	# get request
	r = Message.objects.order_by('-time')[:70]
	res = []
	for msgs in reversed(r) :
		res.append({'id':msgs.id,'user':msgs.user,'msg':msgs.message,'time':msgs.time.strftime('%I:%M:%S %p').lstrip('0'),'gravatar':msgs.gravatar})
	
	data = json.dumps(res)

	
	return HttpResponse(data,content_type="application/json")


def logged_chat_users(request):

	u = ChatUser.objects.filter(is_chat_user=True)

	for j in u:
		elapsed = utcnow() - j.last_accessed
		if elapsed > datetime.timedelta(seconds=35):

			j.is_chat_user = False
			j.save()

	uu = ChatUser.objects.filter(is_chat_user=True)
	

	d = []
	for i in uu:
		d.append({'username': i.username,'gravatar':i.gravatar_url,'id':i.userID})
	data = json.dumps(d)
	 

	return HttpResponse(data,content_type="application/json")


def update_time(request):
	if request.user.username:
		u = request.user.profile


		u.last_accessed = utcnow()
		u.is_chat_user = True
		u.save()

		return HttpResponse('updated')
	return HttpResponse('who are you?')



from django.conf.urls import patterns, url

from djangoChat import views

urlpatterns = patterns('',
    url(r'^$', views.index, name='index'),
    url(r'^login/$',views.login,name='login'),
    url(r'^logout/$',views.logout,name='logout'),
    url(r'^api/$',views.chat_api,name='chat_api'),
    url(r'^api/users/$',views.logged_chat_users,name='logged_chat_users'),
    url(r'^api/users/update/$',views.update_time,name='update_time')
)


import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-ajax-chat',
    version='0.2',
    packages=['djangoChat'],
    include_package_data=True,
    license='MIT License',  
    description='A simple Django ajax chat.',
    long_description=README,
    url='https://github.com/sharan01/django-ajax-chat',
    author='sharan salokye',
    author_email='salokye@sharan.co',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # replace these appropriately if you are using Python 3
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)





from django.db import models
from datetime import datetime
from django.contrib.auth.signals import user_logged_in, user_logged_out  
from django.contrib.auth.models import User
import urllib, hashlib, binascii

class Message(models.Model):
	user = models.CharField(max_length=200)
	message = models.TextField(max_length=200)
	time = models.DateTimeField(auto_now_add=True)
	gravatar = models.CharField(max_length=300)
	def __unicode__(self):
		return self.user
	# def save(self):
	# 	if self.time == None:
	# 		self.time = datetime.now()
	# 	super(Message, self).save()




def generate_avatar(email):
	a = "http://www.gravatar.com/avatar/"
	a+=hashlib.md5(email.lower()).hexdigest()
	a+='?d=identicon'
	return a
def hash_username(username):
	a = binascii.crc32(username)
	return a
class ChatUser(models.Model):
	user = models.OneToOneField(User)
	userID =  models.IntegerField()
	username = models.CharField(max_length=300)
	is_chat_user = models.BooleanField(default=False)
	gravatar_url = models.CharField(max_length=300)
	last_accessed = models.DateTimeField(auto_now_add=True)
	
User.profile = property(lambda u: ChatUser.objects.get_or_create(user=u,defaults={'gravatar_url':generate_avatar(u.email),'username':u.username,'userID':hash_username(u.username)})[0])
